// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.annotatedlineeditor;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import org.apache.log4j.Logger;
import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.Document;
import org.eclipse.jface.text.DocumentEvent;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.IRegion;
import org.eclipse.jface.text.ITypedRegion;
import org.eclipse.jface.text.Region;
import org.eclipse.jface.text.TypedRegion;
import org.eclipse.jface.text.rules.FastPartitioner;
import org.eclipse.jface.text.rules.RuleBasedPartitionScanner;

/**
 * Separates each line into partitions for the main text - usually the
 * object's title cache - and supplemental text. The supplemental text
 * partition can then be given a different color and / or font style by 
 * the editor's presentation reconciler. 
 * 
 * @author p.ciardelli
 * @created 28.10.2009
 * @version 1.0
 * @deprecated
 */
public class AnnotatedLineEditorPartitioner extends FastPartitioner {
	private static final Logger logger = Logger
			.getLogger(AnnotatedLineEditorPartitioner.class);
	
	private Document document;
	private LineAnnotationModel model;
	private ILineDisplayStrategy lineDisplayStrategy;
	
	/**
	 * @param scanner
	 * @param legalContentTypes
	 */
	public AnnotatedLineEditorPartitioner(Document document, LineAnnotationModel model, ILineDisplayStrategy lineDisplayStrategy) {
		super(new RuleBasedPartitionScanner(), 
				new String[] {ILineDisplayStrategy.SUPPLEMENTAL_TEXT});
		
		this.document = document;
		this.model = model;
		this.lineDisplayStrategy = lineDisplayStrategy;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.text.rules.FastPartitioner#computePartitioning(int, int, boolean)
	 */
	@Override
	public ITypedRegion[] computePartitioning(int offset,
			int length, boolean includeZeroLengthPartitions) {
		List<ITypedRegion> regions = new ArrayList<ITypedRegion>(); 
		try {
			int lines = document.getNumberOfLines(offset, length);
			int startLine = document.getLineOfOffset(offset);
			
			// Iterate through all affected lines, separating into regions of DEFAULT_CONTENT_TYPE and SUPPLEMENTAL_TEXT
			for (int i = startLine; i < startLine + lines; i++) {
				LineAnnotation annotation = (LineAnnotation) ((LineAnnotationModel) model).getAnnotationAtLine(i, document);
				if (annotation != null) {
					IRegion lineInfo = document.getLineInformation(i);
					String lineText = document.get(lineInfo.getOffset(), lineInfo.getLength());
					
					String supplementalText = lineDisplayStrategy.getSupplementalText(annotation.getEntity());
					if (supplementalText == null || supplementalText.equals("")) {
						continue;
					}
					if (lineText.contains(supplementalText)) {
						int startSupplementalText = lineText.indexOf(supplementalText) + lineInfo.getOffset();
						
						if (startSupplementalText > lineInfo.getOffset()) {
							regions.add(new TypedRegion(lineInfo.getOffset(), startSupplementalText - lineInfo.getOffset(), 
															IDocument.DEFAULT_CONTENT_TYPE));
							logger.warn("Non supp at " + lineInfo.getOffset() + ", length " + (startSupplementalText - lineInfo.getOffset()));
						}														
						regions.add(new TypedRegion(startSupplementalText, supplementalText.length(), 
														ILineDisplayStrategy.SUPPLEMENTAL_TEXT));
						logger.warn("Supp at " + startSupplementalText + ", length " + supplementalText.length());
						if (startSupplementalText + supplementalText.length() < lineInfo.getOffset() + lineInfo.getLength()) {
							regions.add(new TypedRegion(startSupplementalText + supplementalText.length(), 
															lineInfo.getLength() - startSupplementalText - supplementalText.length(), 
															IDocument.DEFAULT_CONTENT_TYPE));
						}									
					}
					
				}
			}
		} catch (BadLocationException e) {
			return super.computePartitioning(offset, length, includeZeroLengthPartitions);
		}
		return (ITypedRegion[]) regions.toArray(new ITypedRegion[regions.size()]);
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.text.rules.FastPartitioner#documentChanged2(org.eclipse.jface.text.DocumentEvent)
	 */
	@Override
	public IRegion documentChanged2(DocumentEvent e) {
		computePartitioning(e.getOffset(), e.getLength());
		return new Region(e.getOffset(), e.getLength());
//		return super.documentChanged2(e);
	}
}
