// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.bulkeditor;

import java.util.Comparator;

import org.apache.log4j.Logger;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.ToolBar;
import org.eclipse.swt.widgets.ToolItem;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.swt.IFocusService;

import eu.etaxonomy.cdm.api.service.config.IIdentifiableEntityServiceConfigurator;
import eu.etaxonomy.cdm.api.service.config.impl.IdentifiableServiceConfiguratorImpl;
import eu.etaxonomy.taxeditor.bulkeditor.input.BulkEditorInput;
import eu.etaxonomy.taxeditor.model.Resources;

/**
 * @author p.ciardelli
 * @author e.-m.lee
 * @created 17.08.2009
 * @version 1.0
 */
public class BulkEditorSearchComposite extends Composite {
	private static final Logger logger = Logger.getLogger(BulkEditorSearchComposite.class);
	
	private IEditorPart editor;
	private IBulkEditorSortMenuProvider menuProvider;
	private Menu sortMenu;
	private Text text;
	private static final String DEFAULT_TEXT = "Use \"*\" for wildcard searching";
	public Object ORDER_BY = new Object();
	private ToolItem toolItem;
	
	/**
	 * @param parent
	 * @param style
	 */
	public BulkEditorSearchComposite(IEditorPart editor, Composite parent, int style) {
		super(parent, style);
		this.editor = editor;
		this.menuProvider = new BulkEditorSortMenuProvider();

		createControl();
	}

	/*
	 * Creates the search control.
	 */
	protected void createControl() {
		createLayout();
		createSearchTextField();
		createToolBar();
		registerAtFocusService();
	    setSearchEnabled(false);
	}


	/**
	 * Handles focus changes for the textfield.
	 */
	private void registerAtFocusService() {
		IFocusService focusService = 
			(IFocusService) PlatformUI.getWorkbench().getService(IFocusService.class);
		if (focusService != null) {
			focusService.addFocusTracker(text, "bulkeditor.textControlId");
		}
	}


	/**
	 * Creates the search toolbar.
	 */
	private void createToolBar() {
		final ToolBar toolBar = new ToolBar(this, SWT.NULL);
		
		toolItem = new ToolItem(toolBar, SWT.DROP_DOWN | SWT.BORDER);
		toolItem.setText("Search");
		
		DropdownMenu dropdownMenu = new DropdownMenu(toolItem);
	    toolItem.addSelectionListener(dropdownMenu);
	}


	/**
	 * Creates the search textfield.
	 */
	private void createSearchTextField() {
		text = new Text(this, SWT.BORDER | SWT.SINGLE | SWT.FULL_SELECTION);
		text.setText(DEFAULT_TEXT);
		text.setForeground(Resources.getColor(Resources.SEARCH_VIEW_FOREGROUND));
		
		text.addFocusListener(new FocusListener() {

			public void focusGained(FocusEvent e) {
				text.setForeground(Resources.getColor(Resources.SEARCH_VIEW_FOCUS));
				if (DEFAULT_TEXT.equals(text.getText())) {
					text.setText("");
				}
			}

			public void focusLost(FocusEvent e) {
				if (text.getText() == "") {
					text.setForeground(Resources.getColor(Resources.SEARCH_VIEW_FOREGROUND));
					text.setText(DEFAULT_TEXT);			
				    setSearchEnabled(false);		
				} else {
				    setSearchEnabled(true);					
				}
			}
		});
		
		text.addKeyListener(new KeyAdapter() {
			/* (non-Javadoc)
			 * @see org.eclipse.swt.events.KeyAdapter#keyReleased(org.eclipse.swt.events.KeyEvent)
			 */
			@Override
			public void keyReleased(KeyEvent e) {
				if (e.keyCode == SWT.CR) {
					updateEditorInput();
				}
			}
		});
		
		GridData gridData = new GridData(SWT.FILL, SWT.TOP, true, false);
		text.setLayoutData(gridData);
	}


	/**
	 * Creates the search layout.
	 */
	private void createLayout() {
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 3;
		gridLayout.marginHeight = 0;
		gridLayout.marginWidth = 12;
		setLayout(gridLayout);
	}
	
	/*
	 * Toggles the availability of the search toolItem.
	 */
	private void setSearchEnabled(boolean enabled) {
//		toolItem.setEnabled(enabled);		
	}
	
	/*
	 * Shows the results of the search.
	 */
	private void updateEditorInput() {
		
		String searchString = getSearchString();
		
		if(!DEFAULT_TEXT.equals(searchString.trim()) && searchString.length() > 0){
			// update query in IEditorInput
			IEditorInput input = editor.getEditorInput();
			if (input instanceof BulkEditorInput) {
				((BulkEditorInput) input).setQuery(new BulkEditorQuery(getSearchString(), getComparator()));
			}
		}
	}
	
	/*
	 * Handles drop down menu selection.
	 */
	class DropdownMenu extends SelectionAdapter {

		/**
		 * @param dropdown
		 */
		public DropdownMenu(ToolItem dropdown) {
			sortMenu = new Menu(dropdown.getParent().getShell());
			
			MenuItem menuItem = new MenuItem(sortMenu, SWT.PUSH);
		    menuItem.setText("Order by:");
		    menuItem.setData(ORDER_BY);
			new MenuItem(sortMenu, SWT.SEPARATOR);

			menuProvider.fillMenu(editor.getEditorInput(), sortMenu);
			
			SelectionListener listener = new SortListSelectionListener();
			
			for (MenuItem item : sortMenu.getItems()) {
				if (item.getData() instanceof Comparator) {
					item.addSelectionListener(listener);
				}
			}
		}

		/* (non-Javadoc)
		 * @see org.eclipse.swt.events.SelectionAdapter#widgetSelected(org.eclipse.swt.events.SelectionEvent)
		 */
		public void widgetSelected(SelectionEvent event) {
			if (event.detail == SWT.ARROW) {
				ToolItem item = (ToolItem) event.widget;
				Rectangle rect = item.getBounds();
				Point pt = item.getParent().toDisplay(new Point(rect.x, rect.y));
				sortMenu.setLocation(pt.x, pt.y + rect.height);
				sortMenu.setVisible(true);
		    } else {
		    	updateEditorInput();
		    }
		}
	}

	/*
	 * Handles search configuration selection.
	 */
	class SortListSelectionListener extends SelectionAdapter {
		
		public void widgetSelected(SelectionEvent e) {
			for (MenuItem item : sortMenu.getItems()) {
				if (item.getData() instanceof Comparator) {
					if (item.equals(e.getSource())) {
						item.setSelection(true);
					} else {
						item.setSelection(false);
					}
				}
			}				
		}
	}
	
	/*
	 * Returns the current string in the search textfield.
	 * @return the content of the textfield
	 */
	public String getSearchString() {
		return text.getText().trim();
	}
	
	/*
	 * 
	 */
	public Comparator getComparator() {
		Comparator comparator;
		for (MenuItem item : sortMenu.getItems()) {
			if (item.getSelection() == true && item.getData() instanceof Comparator) {
				return (Comparator) item.getData();
			}
		}
		return null;
	};
	
	/*
	 * 
	 */
	class BulkEditorQuery implements IBulkEditorQuery {
		
		private String searchString;
		private Comparator comparator;
		private IIdentifiableEntityServiceConfigurator searchConfigurator;

		BulkEditorQuery (String searchString, Comparator comparator) {
			this.searchString = searchString;
			this.comparator = comparator;
			
			searchConfigurator = IdentifiableServiceConfiguratorImpl.NewInstance();
			searchConfigurator.setTitleSearchString(searchString);
		}

		/* (non-Javadoc)
		 * @see eu.etaxonomy.taxeditor.bulkeditor.IBulkEditorQuery#getComparator()
		 */
		public Comparator getComparator() {
			return comparator;
		}

		/* (non-Javadoc)
		 * @see eu.etaxonomy.taxeditor.bulkeditor.IBulkEditorQuery#getSearchString()
		 */
		public String getSearchString() {
			return searchString;
		}

		/* (non-Javadoc)
		 * @see eu.etaxonomy.taxeditor.bulkeditor.IBulkEditorQuery#getSearchConfigurator()
		 */
		public IIdentifiableEntityServiceConfigurator getSearchConfigurator() {
			return searchConfigurator;
		}
	}
}