// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.bulkeditor.handler;

import java.beans.PropertyChangeEvent;
import java.util.Iterator;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.jface.text.TextSelection;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.widgets.Event;
import org.eclipse.ui.handlers.HandlerUtil;

import eu.etaxonomy.cdm.model.common.AnnotatableEntity;
import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.common.Marker;
import eu.etaxonomy.cdm.model.common.MarkerType;
import eu.etaxonomy.taxeditor.annotatedlineeditor.LineAnnotation;
import eu.etaxonomy.taxeditor.annotatedlineeditor.LineAnnotationModel;
import eu.etaxonomy.taxeditor.annotatedlineeditor.LineSelection;
import eu.etaxonomy.taxeditor.bulkeditor.BulkEditor;

/**
 * @author p.ciardelli
 * @created 20.10.2009
 * @version 1.0
 */
public class SetMarkerFlagTrueHandler extends AbstractHandler {
	private static final Logger logger = Logger
			.getLogger(SetMarkerFlagTrueHandler.class);

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.IHandler#execute(org.eclipse.core.commands.ExecutionEvent)
	 */
	public Object execute(ExecutionEvent event) throws ExecutionException {
		
		Object data = ((Event) event.getTrigger()).data;
		
		if (HandlerUtil.getCurrentSelection(event) instanceof TextSelection 
					&& HandlerUtil.getActiveEditor(event) instanceof BulkEditor && data instanceof MarkerType) {
			TextSelection selection = (TextSelection) HandlerUtil.getCurrentSelection(event);
			BulkEditor editor = (BulkEditor) HandlerUtil.getActiveEditor(event);
			
			LineAnnotationModel model = (LineAnnotationModel) editor.getDocumentProvider().getAnnotationModel(editor.getEditorInput());
			if (model != null) {
				Iterator iter = model.getAnnotationIterator(selection.getOffset(), selection.getLength(), true, true);
				while (iter.hasNext()) {
					Object next = iter.next();
					if (next instanceof LineAnnotation) {
						Object entity = ((LineAnnotation) next).getEntity();
						createOrUpdateMarker((AnnotatableEntity) entity, (MarkerType) data, true);
						((CdmBase) entity).firePropertyChange(new PropertyChangeEvent(entity, "", null, null));
					}
				}
			}
		}
		return null;
	}

	/**
	 * @param entity
	 * @param publish
	 * @param b
	 */
	private void createOrUpdateMarker(AnnotatableEntity entity,
			MarkerType markerType, boolean flag) {
		for (Marker marker : entity.getMarkers()) {
			if (markerType.equals(marker.getMarkerType())) {
				marker.setFlag(flag);
				return;
			}
		}
		entity.addMarker(Marker.NewInstance(markerType, flag));
	}
}
