/**
 * 
 */
package eu.etaxonomy.taxeditor.editor.descriptiontree;

import org.apache.log4j.Logger;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.Viewer;

import eu.etaxonomy.cdm.api.utility.ImagesUtility;
import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.cdm.model.description.TaxonDescription;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.editor.TaxonEditorInput;

/**
 * @author p.ciardelli
 *
 */
public class DescriptionContentProvider implements ITreeContentProvider {
	private static final Logger logger = Logger
			.getLogger(DescriptionContentProvider.class);

	private static final Object[] NO_CHILDREN = new Object[0];
	
	private IDescriptiveInformationProvider infoProvider;

	public DescriptionContentProvider(IDescriptiveInformationProvider infoProvider) {
		this.infoProvider = infoProvider;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ITreeContentProvider#getChildren(java.lang.Object)
	 */
	public Object[] getChildren(Object parentElement) {
		if (parentElement instanceof TaxonEditorInput) {
			Taxon taxon = ((TaxonEditorInput) parentElement).getTaxon();
			return taxon.getDescriptions().toArray();
		}
		if (parentElement instanceof TaxonDescription) {
			if (((TaxonDescription) parentElement).isImageGallery()) {
				return infoProvider.getDescriptionElements((TaxonDescription) parentElement, Feature.IMAGE()).toArray();
			} else {
				return infoProvider.getDescriptionFeatures((TaxonDescription) parentElement).toArray();
			}
		}
		if (parentElement instanceof TaxonDescriptionFeature){
			TaxonDescriptionFeature taxonDescriptionFeature = (TaxonDescriptionFeature) parentElement;
			Feature feature = taxonDescriptionFeature.getFeature();
			TaxonDescription description = taxonDescriptionFeature.getDescription();
			return infoProvider.getDescriptionElements(description, feature).toArray();
		}
		if (parentElement instanceof DescriptionElementBase) {
			DescriptionElementBase descriptionElement = (DescriptionElementBase) parentElement; 
			if (descriptionElement.getInDescription().isImageGallery()) {
				return ImagesUtility.getOrderedImages(descriptionElement).toArray();
			}
		}
		return NO_CHILDREN;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ITreeContentProvider#getParent(java.lang.Object)
	 */
	public Object getParent(Object element) {
		return null;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ITreeContentProvider#hasChildren(java.lang.Object)
	 */
	public boolean hasChildren(Object element) {
		return (getChildren(element).length > 0);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(java.lang.Object)
	 */
	public Object[] getElements(Object inputElement) {
		return getChildren(inputElement);
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.IContentProvider#dispose()
	 */
	public void dispose() {}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(org.eclipse.jface.viewers.Viewer, java.lang.Object, java.lang.Object)
	 */
	public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {}
}