/**
 * 
 */
package eu.etaxonomy.taxeditor.editor.descriptiontree;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.cdm.model.description.TaxonDescription;

/**
 * @author p.ciardelli
 *
 */
public class DescriptiveInformationProvider implements
		IDescriptiveInformationProvider {
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.editor.descriptionnew.IDescriptiveInformationProvider#getDescriptionElements(eu.etaxonomy.cdm.model.description.TaxonDescription, eu.etaxonomy.cdm.model.description.Feature)
	 */
	public List<DescriptionElementBase> getDescriptionElements(
			TaxonDescription description, Feature feature) {
		List<DescriptionElementBase> featureElements = new ArrayList<DescriptionElementBase>();
		
		Set<DescriptionElementBase> elements = description.getElements();
		if (elements != null) {
			for (DescriptionElementBase element : elements) {
				if (element.getFeature().equals(feature)) {
					featureElements.add(element);
				}
			}
		}
		return featureElements;
	}

	/**
	 * TaxonDescriptionFeature objects are cached so that they pass description tree viewer's equality test.
	 */
	private Map<TaxonDescription, Map<Feature, TaxonDescriptionFeature>> descriptionFeatureCache = new HashMap<TaxonDescription, Map<Feature, TaxonDescriptionFeature>>();
		
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.editor.descriptionnew.IDescriptiveInformationProvider#getDescriptionFeatures(eu.etaxonomy.cdm.model.description.TaxonDescription)
	 */
	public List<TaxonDescriptionFeature> getDescriptionFeatures(TaxonDescription description) {
		
		Map<Feature, TaxonDescriptionFeature> featureCache = getFeatureCache(description);
		
		Set<Feature> featuresTemp = new HashSet<Feature>();
		
		Set<DescriptionElementBase> elements = description.getElements();
		if (elements != null) {
			for (DescriptionElementBase element : elements) {
				Feature feature = HibernateProxyHelper.deproxy(element.getFeature(), Feature.class);
				featuresTemp.add(feature);
				
				// Is there already an entry in map for this Feature?
				if (!(featureCache.containsKey(feature))) {
					featureCache.put(feature, new TaxonDescriptionFeature(description, feature));
				}				
			}
		}
		
		// Check whether any features need to be removed, i.e. their elements were deleted
		Set<Feature> keys = featureCache.keySet();
		HashSet<Feature> removeFeatures = new HashSet<Feature>();
		for (Feature feature : keys) {
			if (!featuresTemp.contains(feature)) {
				removeFeatures.add(feature);
			}
		}
		for (Feature feature : removeFeatures) {
			featureCache.remove(feature);
		}
		
		return new ArrayList<TaxonDescriptionFeature>(featureCache.values());
	}

	private Map<Feature, TaxonDescriptionFeature> getFeatureCache(TaxonDescription description) {
		Map<Feature, TaxonDescriptionFeature> featureCache = descriptionFeatureCache.get(description);
		if (featureCache == null) {
			featureCache = new HashMap<Feature, TaxonDescriptionFeature>();
			descriptionFeatureCache.put(description, featureCache);
		}
		return featureCache;
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.editor.descriptiontree.IDescriptiveInformationProvider#getDescriptionFeature(eu.etaxonomy.cdm.model.description.DescriptionElementBase)
	 */
	public TaxonDescriptionFeature getDescriptionFeature(DescriptionElementBase element) {
		
		TaxonDescription description = (TaxonDescription) element.getInDescription();
		Feature feature = HibernateProxyHelper.deproxy(element.getFeature(), Feature.class);
		
		Map<Feature, TaxonDescriptionFeature> featureCache = getFeatureCache(description);
		TaxonDescriptionFeature tdf = featureCache.get(feature);
		if (tdf == null) {
			tdf = new TaxonDescriptionFeature(description, feature);
			featureCache.put(feature, tdf);
		}
		return tdf;
	}
}
