/**
 * 
 */
package eu.etaxonomy.taxeditor.editor.descriptiontree;

import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.forms.IManagedForm;
import org.eclipse.ui.forms.editor.FormEditor;
import org.eclipse.ui.forms.editor.FormPage;
import org.eclipse.ui.forms.widgets.ScrolledForm;

import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.taxeditor.editor.IMultiPageTaxonEditorPage;
import eu.etaxonomy.taxeditor.operations.IPostOperationEnabled;

/**
 * @author p.ciardelli
 *
 */
public class TaxonDescriptionTreeEditor extends FormPage implements IMultiPageTaxonEditorPage, IPostOperationEnabled {

	private static final String ID = "eu.etaxonomy.taxeditor.descriptiontreeeditor";
	
	private DescriptionMasterDetailsBlock block;

	private boolean isRedrawing;

	/**
	 * @param editor
	 */
	public TaxonDescriptionTreeEditor(FormEditor editor) {
		super(editor, "id","title");
		block = new DescriptionMasterDetailsBlock(this);
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.forms.editor.FormPage#init(org.eclipse.ui.IEditorSite, org.eclipse.ui.IEditorInput)
	 */
	@Override
	public void init(IEditorSite site, IEditorInput input) {
		ISelectionProvider selectionProvider = null;
		if (getSite() != null && getSite().getSelectionProvider() != null) {
			selectionProvider = getSite().getSelectionProvider();
			site.setSelectionProvider(selectionProvider);
		}
		super.init(site, input);
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.forms.editor.FormPage#createFormContent(org.eclipse.ui.forms.IManagedForm)
	 */
	protected void createFormContent(final IManagedForm managedForm) {
		final ScrolledForm scrolledForm = managedForm.getForm();
		scrolledForm.setText("Descriptions"); 
		managedForm.getToolkit().decorateFormHeading(scrolledForm.getForm());
		block.createContent(managedForm);
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.forms.editor.FormPage#getId()
	 */
	@Override
	public String getId() {
		return ID;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.forms.editor.FormPage#isEditor()
	 */
	@Override
	public boolean isEditor() {
		return true;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.operations.IPostOperationEnabled#postOperation(eu.etaxonomy.cdm.model.common.CdmBase)
	 */
	public boolean postOperation(CdmBase objectAffectedByOperation) {
		getManagedForm().dirtyStateChanged();
		block.refreshTree(objectAffectedByOperation);
		return true;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.editor.IMultiPageTaxonEditorPage#redraw()
	 */
	public boolean redraw() {
		return redraw(true);
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.editor.IMultiPageTaxonEditorPage#redraw(boolean)
	 */
	public boolean redraw(boolean focus) {
		isRedrawing = true;
		block.refreshTree(null);
		isRedrawing = false;
		this.setFocus();
		return true;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.editor.IMultiPageTaxonEditorPage#isRedrawing()
	 */
	public boolean isRedrawing() {
		return isRedrawing;
	}
}
