// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.descriptiontree.handler;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.common.NotDefinedException;
import org.eclipse.core.commands.operations.IUndoableOperation;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.TreePath;
import org.eclipse.jface.viewers.TreeSelection;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.forms.editor.FormEditor;
import org.eclipse.ui.handlers.HandlerUtil;

import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.cdm.model.description.TaxonDescription;
import eu.etaxonomy.cdm.model.media.ImageFile;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.TaxonEditorInput;
import eu.etaxonomy.taxeditor.operations.DeleteDescriptionElementOperation;
import eu.etaxonomy.taxeditor.operations.DeleteImageOperation;
import eu.etaxonomy.taxeditor.operations.DeleteTaxonDescriptionOperation;
import eu.etaxonomy.taxeditor.operations.IPostOperationEnabled;

/**
 * @author n.hoffmann
 * @created Jan 19, 2010
 * @version 1.0
 */
public class DeleteHandler extends AbstractHandler {
	private static final Logger logger = Logger.getLogger(DeleteHandler.class);

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.IHandler#execute(org.eclipse.core.commands.ExecutionEvent)
	 */
	public Object execute(ExecutionEvent event) throws ExecutionException {
		IEditorPart editor = HandlerUtil.getActiveEditor(event);
		
		if (editor instanceof FormEditor) {
			editor = ((FormEditor) editor).getActiveEditor();
		}
		
		IEditorInput input = editor.getEditorInput();
		
		if (input instanceof TaxonEditorInput) {
			Taxon taxon = ((TaxonEditorInput) input).getTaxon();
			
			ISelection selection = EditorUtil.getCurrentSelection();
			
			Object deleteElement = ((TreeSelection) selection).getFirstElement();
			
			IUndoableOperation operation = null;
			String commandName = null;
			
			try {
				commandName = event.getCommand().getName();
			} catch (NotDefinedException e) {
				logger.error(e);
				throw new RuntimeException(e);
			}			
			
			// TODO use undo context specific to editor
			if (deleteElement instanceof DescriptionElementBase) {
				DescriptionElementBase descriptionElement = (DescriptionElementBase) deleteElement;
				
				operation = new DeleteDescriptionElementOperation(commandName, 
						EditorUtil.getUndoContext(), taxon, descriptionElement, (IPostOperationEnabled) editor);
			}
			else if (deleteElement instanceof ImageFile) {
				ImageFile imageFile = (ImageFile) deleteElement;
				TreePath[] paths = ((TreeSelection) selection).getPaths();
				TaxonDescription description = (TaxonDescription) (paths[0]).getFirstSegment();
				
				operation = new DeleteImageOperation(commandName, 
						EditorUtil.getUndoContext(), taxon, description, imageFile, (IPostOperationEnabled) editor);
			}
			else if(deleteElement instanceof TaxonDescription){
				
				TaxonDescription description = (TaxonDescription) deleteElement;
				
				operation = new DeleteTaxonDescriptionOperation(commandName, 
						EditorUtil.getUndoContext(), taxon, description, (IPostOperationEnabled) editor);
				EditorUtil.executeOperation(operation);
			}
			
			EditorUtil.executeOperation(operation);
			
		}		
		return null;
	}
}
