// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.name.handler;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.IHandler;
import org.eclipse.core.commands.common.NotDefinedException;
import org.eclipse.core.commands.operations.IUndoableOperation;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.ui.handlers.HandlerUtil;

import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonRelationshipType;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.Page;
import eu.etaxonomy.taxeditor.editor.name.TaxonNameEditor;
import eu.etaxonomy.taxeditor.operations.ChangeConceptRelationshipTypeOperation;
import eu.etaxonomy.taxeditor.operations.ChangeSynonymToMisapplicationOperation;
import eu.etaxonomy.taxeditor.propertysheet.name.ConceptPropertySource;
import eu.etaxonomy.taxeditor.propertysheet.name.SynonymPropertySource;
import eu.etaxonomy.taxeditor.propertysheet.name.TaxonBasePropertySource;

/**
 * @author n.hoffmann
 * @created 21.04.2009
 * @version 1.0
 */
public class ChangeToMisapplicationHandler extends AbstractHandler implements
		IHandler {
	private static final Logger logger = Logger
			.getLogger(ChangeToMisapplicationHandler.class);

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.IHandler#execute(org.eclipse.core.commands.ExecutionEvent)
	 */
	public Object execute(ExecutionEvent event) throws ExecutionException {
		TaxonNameEditor editor = (TaxonNameEditor) EditorUtil.getActiveEditorPage(
				Page.NAME);

		ISelection menuSelection = HandlerUtil.getActiveMenuSelection(event);
		
		TaxonBasePropertySource propertySource = (TaxonBasePropertySource) ((StructuredSelection) menuSelection).getFirstElement();
		
		IUndoableOperation operation = null;
		try {
			if(propertySource instanceof ConceptPropertySource){
				Taxon concept = (Taxon) propertySource.getTaxonBase();
				operation = new ChangeConceptRelationshipTypeOperation(event.getCommand().getName(), 
										editor.getUndoContext(), editor.getTaxon(), concept, TaxonRelationshipType.MISAPPLIED_NAME_FOR(), editor);
			}
			if(propertySource instanceof SynonymPropertySource){
				Synonym synonym = (Synonym) propertySource.getTaxonBase();
				operation = new ChangeSynonymToMisapplicationOperation(event.getCommand().getName(), 
						editor.getUndoContext(), editor.getTaxon(), synonym, editor);
			}
			
			EditorUtil.executeOperation(operation);
		} catch (NotDefinedException e) {
			logger.warn("Command name not set");
		}
		
		return null;
	}
}
