// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.forms;

import org.apache.log4j.Logger;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;

/**
 * @author n.hoffmann
 * @created Nov 18, 2009
 * @version 1.0
 */
public class ToggleableTextField extends AbstractFormComposite implements SelectionListener {


	private static final Logger logger = Logger
			.getLogger(ToggleableTextField.class);
	private TextWithLabelComposite text_cache;
	private Composite container;
	private Button button_toggle;
	
	private boolean state;
	
	/**
	 * @param parent
	 * @param listener
	 * @param style
	 */
	public ToggleableTextField(Composite parent,
			IPropertyChangeListener listener, String label, String initialText, boolean initialState, int style) {
		super(parent, listener, style);
		
				
		container = toolkit.createComposite(this);
		container.setLayout(new GridLayout(2, false));
		container.setLayoutData(FILL_GRID_HORIZONTALLY());
		
		text_cache = toolkit.createTextWithLabelComposite(container, label, initialText, this, SWT.NULL);
		text_cache.setLayoutData(FILL_GRID_HORIZONTALLY());
		button_toggle = toolkit.createButton(container, "Toggle", SWT.TOGGLE);
		button_toggle.setLayoutData(new GridData(SWT.RIGHT, SWT.END, false, false));
		button_toggle.addSelectionListener(this);
		
		setState(initialState);
	}

	public void setText(String text){
		if(text != null){
			text_cache.setText(text);
		}
	}
	
	public String getText(){
		return text_cache.getText();
	}
	
	/**
	 * @param initialState
	 */
	public void setState(boolean state) {
		this.state = state;
		text_cache.setEnabled(state);
		button_toggle.setSelection(state);
	}

	public boolean getState(){
		return state;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.swt.events.SelectionListener#widgetSelected(org.eclipse.swt.events.SelectionEvent)
	 */
	public void widgetSelected(SelectionEvent e) {
		setState(button_toggle.getSelection());
	}
	
	public void widgetDefaultSelected(SelectionEvent e) {}
}
