/**
 * 
 */
package eu.etaxonomy.taxeditor.forms.entitysections;

import java.util.Set;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;

import eu.etaxonomy.cdm.model.common.IdentifiableEntity;
import eu.etaxonomy.cdm.model.media.Rights;

/**
 * @author nho
 *
 */
public class RightsSection extends AbstractEntitySetSection<IdentifiableEntity> {

	/**
	 * @param parent
	 * @param entity
	 * @param style
	 */
	public RightsSection(Composite parent, IdentifiableEntity entity,
			IPropertyChangeListener listener, int style) {
		super(parent, entity, "Rights", listener, style);
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.forms.AbstractEntitySetSection#createDynamicContents()
	 */
	@Override
	protected void createDynamicContents() {
		Set<Rights> rights = null;
		if(getEntity() != null){
			rights = getEntity().getRights();
		}
		
		if(rights == null || rights.size() == 0){
			toolkit.createLabel(container, "No rights yet.");
		}else{
			for(Rights right : rights){
				new RightsComposite(container, right, new RemoveListener(right), this, SWT.NULL);
			}
		}
	}
	
	private class RemoveListener extends SelectionAdapter{
		private Rights rights;
		
		public RemoveListener(Rights rights){
			this.rights = rights;
		}
		
		/* (non-Javadoc)
		 * @see org.eclipse.swt.events.SelectionAdapter#widgetSelected(org.eclipse.swt.events.SelectionEvent)
		 */
		@Override
		public void widgetSelected(SelectionEvent e) {
			getEntity().removeRights(rights);
			internalUpdateSection(true);
		}
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.forms.AbstractEntitySetSection#getAddAction()
	 */
	@Override
	protected IAction getAddAction() {
		Action addRights = new Action("add", Action.AS_PUSH_BUTTON){
			/* (non-Javadoc)
			 * @see org.eclipse.jface.action.Action#run()
			 */
			@Override
			public void run() {
				Rights rights = Rights.NewInstance();
				getEntity().addRights(rights);
				internalUpdateSection(true);
				if(! getSection().isExpanded())
					getSection().setExpanded(true);
				internalUpdateSection(true);
			}
		};
		addRights.setToolTipText("Create a new marker");
		return addRights;
	}

}
