/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.propertysheet.reference;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.DialogCellEditor;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.views.properties.PropertyDescriptor;

import eu.etaxonomy.cdm.model.reference.IReferenceBase;
import eu.etaxonomy.cdm.model.reference.ReferenceBase;
import eu.etaxonomy.cdm.model.reference.ReferenceType;
import eu.etaxonomy.taxeditor.dialogs.filteredSelection.FilteredReferenceSelectionDialog;

/**  
 * @author p.ciardelli
 * @created 19.05.2008
 * @version 1.0
 */
abstract public class ReferenceSearchDescriptor extends PropertyDescriptor {
		
	private ReferenceBase existingReference;
	private ReferenceType[] requiredTypes;
	
	public ReferenceSearchDescriptor(Object id, String displayName, ReferenceBase reference) {
		super(id, displayName);
		
		this.existingReference = existingReference;
	}

	public ReferenceSearchDescriptor(Object id, String displayName, ReferenceBase existingReference, ReferenceType[] requiredTypes) {
		super(id, displayName);
		
		this.existingReference = existingReference;
		this.requiredTypes = requiredTypes;
	}
	

	public CellEditor createPropertyEditor(Composite parent) {
        CellEditor editor = new DialogCellEditor(parent) {

			protected Object openDialogBox(
					Control cellEditorWindow) {			

		        ReferenceBase reference = FilteredReferenceSelectionDialog.selectReference(cellEditorWindow.getShell(), existingReference);
				if (reference != null) {
					if (requiredTypes != null && !isRequiredType(reference)) {
						MessageDialog.openInformation(PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell(), 
								"Invalid Reference", "'" + reference.getTitle() + 
								"' is not one of the following allowed reference types:\n\n" + getTabbedRequiredTypeString());
					} else {
						saveReference(reference);
						return new ReferencePropertySource(reference);
					}
				}
				return null;
				
			}
        };
        if (getValidator() != null) {
			editor.setValidator(getValidator());
		}
        return editor;
    }
        
    /**
	 * @return
	 */
	private String getTabbedRequiredTypeString() {
		String str = ""; 
		for (ReferenceType requiredType : requiredTypes) {
			str += "\t" + requiredType.getMessage() + "\n"; 
		}		 
		return str;
	}

	/**
     * 
	 * @param reference
	 * @return
	 */
	protected boolean isRequiredType(ReferenceBase reference) {
		for (ReferenceType requiredType : requiredTypes) {
			if (reference.getType().equals(requiredType)) {
				return true;
			}
		}
		return false;
	}

	abstract protected void saveReference(ReferenceBase reference);	
}
