// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.navigation.handler;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.IHandler;
import org.eclipse.core.commands.common.NotDefinedException;
import org.eclipse.core.commands.operations.IUndoableOperation;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.TreeSelection;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorReference;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.handlers.HandlerUtil;

import eu.etaxonomy.cdm.model.taxon.ITreeNode;
import eu.etaxonomy.cdm.model.taxon.TaxonNode;
import eu.etaxonomy.taxeditor.editor.TaxonEditorInput;
import eu.etaxonomy.taxeditor.navigation.NavigationUtil;
import eu.etaxonomy.taxeditor.navigation.navigator.TaxonNavigator;
import eu.etaxonomy.taxeditor.operations.DeleteTreeNodeOperation;

/**
 * @author n.hoffmann
 * @created 06.04.2009
 * @version 1.0
 */
public class DeleteTaxonNodeHandler extends AbstractHandler implements IHandler{
	private static final Logger logger = Logger
			.getLogger(DeleteTaxonNodeHandler.class);
	private IWorkbenchPage activePage;
	private TaxonNavigator taxonNavigator;

	public Object execute(ExecutionEvent event) throws ExecutionException {
		
		// Prompt user for confirmation
		if(! MessageDialog.openConfirm(HandlerUtil.getActiveShell(event), "Confirm Deletion", "Are you sure you want to delete the selected taxa?")){
			return null;
		}
		
		activePage = HandlerUtil.getActiveWorkbenchWindow(event).getActivePage();
		
		try {
			taxonNavigator = (TaxonNavigator) NavigationUtil.getView(TaxonNavigator.ID);
		} catch (PartInitException e) {
			logger.error(e);
			throw new RuntimeException(e);
		}
		
		TreeSelection selection = (TreeSelection) HandlerUtil.getCurrentSelection(event);
		
		Iterator selectionIterator = selection.iterator();
		Set<ITreeNode> treeNodes = new HashSet<ITreeNode>();
		
		while (selectionIterator.hasNext()){
			Object object = selectionIterator.next();
			if(object instanceof ITreeNode)
				treeNodes.add((ITreeNode) object);
		}
			
		IUndoableOperation operation = null;
		try {
			operation = new DeleteTreeNodeOperation(event.getCommand().getName(), NavigationUtil.getUndoContext(), treeNodes, taxonNavigator, taxonNavigator);
			
			IStatus status = NavigationUtil.executeOperation(operation);
			
			
			// FIXME is there a better place for this code?
			if (status == Status.OK_STATUS){
				for (ITreeNode treeNode : treeNodes){
					if(treeNode instanceof TaxonNode) {
						closeObsoleteEditor((TaxonNode) treeNode);
					}
				}
			}
		
		} catch (NotDefinedException e) {
			logger.warn("Command name not set");
		}
		return null;
	}
	
	private void closeObsoleteEditor(TaxonNode taxonNode){
		for (IEditorReference ref : activePage.getEditorReferences()) {
			try {
				IEditorInput input = ref.getEditorInput(); 
				if (input instanceof TaxonEditorInput) {
					TaxonNode node = ((TaxonEditorInput) input).getTaxonNode();
					if (taxonNode.equals(node)) {
						activePage.closeEditor(ref.getEditor(false), false);
					}
				}
			} catch (PartInitException e) {
				continue;
			}
		}
	}
}
