// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.navigation.navigator;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.commands.operations.IUndoableOperation;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.util.LocalSelectionTransfer;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.TreeSelection;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.swt.dnd.TransferData;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.navigator.CommonDropAdapter;
import org.eclipse.ui.navigator.CommonDropAdapterAssistant;

import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.taxon.ITreeNode;
import eu.etaxonomy.cdm.model.taxon.TaxonNode;
import eu.etaxonomy.taxeditor.navigation.NavigationUtil;
import eu.etaxonomy.taxeditor.operations.IPostOperationEnabled;
import eu.etaxonomy.taxeditor.operations.MoveTaxonOperation;

/**
 * @author p.ciardelli
 * @created 03.06.2009
 * @version 1.0
 */
public class TaxonDropAdapterAssistant extends CommonDropAdapterAssistant implements IPostOperationEnabled{
	private static final Logger logger = Logger
			.getLogger(TaxonDropAdapterAssistant.class);

	public static final String ID = "eu.etaxonomy.taxeditor.navigation.navigator.dropassistant"; //$NON-NLS-1$

	private ITreeNode target;
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.navigator.CommonDropAdapterAssistant#handleDrop(org.eclipse.ui.navigator.CommonDropAdapter, org.eclipse.swt.dnd.DropTargetEvent, java.lang.Object)
	 */
	@Override
	public IStatus handleDrop(CommonDropAdapter dropAdapter,
			DropTargetEvent dropTargetEvent, Object target) {
		
		ISelection selection = LocalSelectionTransfer.getTransfer().getSelection();
		if (selection instanceof TreeSelection) {
			Object element = ((TreeSelection) selection).getFirstElement();
			if (element instanceof TaxonNode) {
				TaxonNode childTaxonNode = (TaxonNode) element;
				if (target instanceof ITreeNode) {
					ITreeNode targetTreeNode = (ITreeNode) target;
					
					return moveTaxon(childTaxonNode, targetTreeNode);
				}
			}
		}
				
		return Status.CANCEL_STATUS;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.navigator.CommonDropAdapterAssistant#validateDrop(java.lang.Object, int, org.eclipse.swt.dnd.TransferData)
	 */
	@Override
	public IStatus validateDrop(Object target, int operation,
			TransferData transferType) {
		if (target instanceof ITreeNode) {
			return Status.OK_STATUS;
		}
		return Status.CANCEL_STATUS;
	}
	

	/**
	 * @param childTaxonNode
	 * @param parentTaxon
	 * @return
	 */
	private IStatus moveTaxon(TaxonNode childTaxonNode, ITreeNode targetTreeNode) {
		
		TaxonNavigator taxonNavigator;
		try {
			taxonNavigator = (TaxonNavigator) NavigationUtil.getView(TaxonNavigator.ID);
		} catch (PartInitException e) {
			logger.error(e);
			throw new RuntimeException(e);
		}
		
		this.target = targetTreeNode;
		
		if(targetTreeNode instanceof TaxonNode){
		
			TaxonNode targetTaxonNode = (TaxonNode) targetTreeNode;
			
			// Make sure parentTaxon is not the drop target
			if (!childTaxonNode.isTopmostNode() && childTaxonNode.getParent().equals(targetTaxonNode)){
				return Status.CANCEL_STATUS;
			}
			
			// Make sure taxon is not being dropped onto itself
			if (childTaxonNode.equals(targetTaxonNode)) {
				return Status.CANCEL_STATUS;
			}
			
			// Make sure parent taxon does not have unsaved changes
			if (NavigationUtil.isDirty(targetTaxonNode)){
				MessageDialog.openWarning(NavigationUtil.getShell(), "Unsaved Parent Taxon", "There are unsaved " +
				"changes in the parent taxon. Pleas save first.");
				return Status.CANCEL_STATUS;
			}
		}	
		
		IUndoContext workspaceUndoContext = NavigationUtil.getWorkbenchUndoContext();
		if (workspaceUndoContext == null) {
			logger.error("Workspace undo context is null. DND operation cancelled");
			return Status.CANCEL_STATUS;
		}
		


		IUndoableOperation operation = new MoveTaxonOperation
				("Move Taxon", workspaceUndoContext, childTaxonNode, targetTreeNode, this, taxonNavigator);
		NavigationUtil.executeOperation(operation);	

		
		
		logger.info("Moved taxon " + childTaxonNode + " to new parent " + targetTreeNode);
		
		NavigationUtil.selectInNavigator(childTaxonNode, targetTreeNode);
		
		return Status.OK_STATUS;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.operations.IPostOperationEnabled#postOperation(eu.etaxonomy.cdm.model.common.CdmBase)
	 */
	public boolean postOperation(CdmBase objectAffectedByOperation) {
		NavigationUtil.selectInNavigator(objectAffectedByOperation, target);
		return true;
	}
}
