// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.dialogs.filteredSelection;

import java.util.UUID;

import org.apache.log4j.Logger;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.dialogs.IDialogSettings;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Link;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;

import eu.etaxonomy.cdm.model.reference.ReferenceBase;
import eu.etaxonomy.taxeditor.dialogs.NewReferenceInputDialog;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author n.hoffmann
 * @created 04.06.2009
 * @version 1.0
 */
public class FilteredReferenceSelectionDialog extends AbstractFilteredCdmResourceSelectionDialog<ReferenceBase> {
	private static final Logger logger = Logger
			.getLogger(FilteredReferenceSelectionDialog.class);

	private static final String SHOW_AUTHORS = "ShowAuthors"; //$NON-NLS-1$
	
	private Button button_showAuthors;
	private ShowReferencesWithAuthorAction showReferencesWithAuthorAction;
	private String showAuthorsText = "Include authors in reference name";
	private boolean showAuthors = false;

	private AbstractContentProvider contentProviderTmp;

	private ItemsFilter itemsFilterTmp;
	
	/**
	* @param reference 
	 * @param taxon
	* @return
	*/
	public static ReferenceBase selectReference(Shell shell, ReferenceBase reference) {
		FilteredReferenceSelectionDialog dialog = new FilteredReferenceSelectionDialog(shell, 
				"Choose a reference", false, (ReferenceBase) reference);
		return getSelectionFromDialog(dialog);
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.dialogs.filteredSelection.AbstractFilteredCdmResourceSelectionDialog#fillContentProvider(org.eclipse.ui.dialogs.FilteredItemsSelectionDialog.AbstractContentProvider, org.eclipse.ui.dialogs.FilteredItemsSelectionDialog.ItemsFilter, org.eclipse.core.runtime.IProgressMonitor)
	 */
	@Override
	protected void fillContentProvider(AbstractContentProvider contentProvider,
			ItemsFilter itemsFilter, IProgressMonitor progressMonitor)
			throws CoreException {
		super.fillContentProvider(contentProvider, itemsFilter, progressMonitor);
		contentProviderTmp = contentProvider;
		itemsFilterTmp = itemsFilter;
	}
	
	protected void reloadModel(){
		try {
			super.fillContentProvider(contentProviderTmp, itemsFilterTmp, null);
		} catch (CoreException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	
	/**
	* @param shell
	* @param title
	 * @param reference 
	*/
	protected FilteredReferenceSelectionDialog(Shell shell, String title, boolean multi, ReferenceBase reference) {
		super(shell, title, multi, FilteredReferenceSelectionDialog.class.getCanonicalName(), reference);
		showReferencesWithAuthorAction = new ShowReferencesWithAuthorAction();
	}
	
	/* (non-Javadoc)
	* @see org.eclipse.ui.dialogs.FilteredItemsSelectionDialog#createExtendedContentArea(org.eclipse.swt.widgets.Composite)
	*/
	@Override
	protected Control createExtendedContentArea(Composite parent) {
		
		Composite composite = new Composite(parent, SWT.NONE);
		
		composite.setLayout(new GridLayout());
		
		
		button_showAuthors = new Button(composite, SWT.CHECK);
		button_showAuthors.setText(showAuthorsText);
		
		button_showAuthors.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				showReferencesWithAuthorAction.run();
			}
		});
		
		Link link = new Link(composite, SWT.NONE);
		link.setText("Click <A>here</A> to create a new reference.");
		link.addListener (SWT.Selection, new Listener () {
			public void handleEvent(Event event) {
				
				NewReferenceInputDialog dialog = new NewReferenceInputDialog(getShell());
								
				if (dialog.open() == Window.OK) {
					
					ReferenceBase reference = dialog.getReference();
					
					addObjectToModel(reference);
					setPattern(reference);
				}
			}
		});
		return composite;
	}
	
	/*
	 * (non-Javadoc)
	 * @see org.eclipse.ui.dialogs.FilteredItemsSelectionDialog#restoreDialog(org.eclipse.jface.dialogs.IDialogSettings)
	 */
	protected void restoreDialog(IDialogSettings settings) {
		if (settings.get(SHOW_AUTHORS) != null) {
			showAuthors = settings.getBoolean(SHOW_AUTHORS);
		}
		setShowAuthorButtons(showAuthors);
		super.restoreDialog(settings);
		
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.dialogs.FilteredItemsSelectionDialog#storeDialog(org.eclipse.jface.dialogs.IDialogSettings)
	 */
	@Override
	protected void storeDialog(IDialogSettings settings) {
		settings.put(SHOW_AUTHORS, showAuthors);
		
		super.storeDialog(settings);		
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.dialogs.AbstractFilteredCdmResourceSelectionDialog#getPersistentObject(java.util.UUID)
	 */
	@Override
	protected ReferenceBase getPersistentObject(UUID cdmUuid) {
		return CdmStore.getReferenceService().load(cdmUuid);
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.dialogs.AbstractFilteredCdmResourceSelectionDialog#initModel()
	 */
	@Override
	protected void initModel() {
		if(showAuthors){
			model = CdmStore.getReferenceService().getUuidAndTitleCache();
		}else{
			model = CdmStore.getReferenceService().getUuidAndTitle();
		}
	}
	
	private void setShowAuthorButtons(boolean showAuthors){
		button_showAuthors.setSelection(showAuthors);
		showReferencesWithAuthorAction.setChecked(showAuthors);
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.dialogs.FilteredItemsSelectionDialog#fillViewMenu(org.eclipse.jface.action.IMenuManager)
	 */
	@Override
	protected void fillViewMenu(IMenuManager menuManager) {
		super.fillViewMenu(menuManager);
		menuManager.add(showReferencesWithAuthorAction);
	}
	
	private class ShowReferencesWithAuthorAction extends Action{
		public ShowReferencesWithAuthorAction(){
			super(showAuthorsText, IAction.AS_CHECK_BOX);
		}
		
		public void run(){
			showAuthors = showAuthors ? false : true;
			setShowAuthorButtons(showAuthors);
			initModel();
		}
	}
}