/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.operations;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;

import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.cdm.model.media.ImageFile;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.model.ImagesHelper;

/**
 * @author p.ciardelli
 * @created 05.02.2009
 * @version 1.0
 * @author n.hoffmann
 */
public class AddImageToDescriptionElementOperation extends AbstractPostOperation {
	
	private DescriptionElementBase element;
	private ImageFile image;

	/**
	 * @param name
	 * @param undoContext
	 * @param taxon
	 * @param element
	 * @param postOperationEnabled
	 */
	public AddImageToDescriptionElementOperation(String label,
			IUndoContext undoContext, Taxon taxon,
			DescriptionElementBase element, IPostOperationEnabled postOperationEnabled) {
		super(label, undoContext, taxon, postOperationEnabled);
		
		this.element = element;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#execute(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus execute(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {

		monitor.worked(20);
		image = ImagesHelper.addImagePart(element);
		monitor.worked(40);
        
		return postExecute(image);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#redo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus redo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		return execute(monitor, info);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#undo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus undo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		ImagesHelper.removeImage(element, image);
		return postExecute(element);
	}
}
