/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.operations;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;

import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonRelationshipType;
import eu.etaxonomy.taxeditor.store.StoreUtil;

/**
 * @author p.ciardelli
 * @created 26.01.2009
 * @version 1.0
 * @author n.hoffmann
 */
public class CreateConceptRelationOperation extends AbstractPostOperation {
	
	private Taxon concept;
	private TaxonRelationshipType taxonRelationshipType;
	
	public CreateConceptRelationOperation(String label,
			IUndoContext undoContext, Taxon taxon, Taxon concept, TaxonRelationshipType taxonRelationshipType
			, IPostOperationEnabled postOperationEnabled) {
		super(label, undoContext, taxon, postOperationEnabled);
		this.concept = concept;
		this.taxonRelationshipType = taxonRelationshipType;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#execute(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus execute(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		
		// Start the main progress monitor.
        IProgressMonitor newMonitor = StoreUtil.startMainMonitor(monitor,"Creating Concept Relationship", 2);
        
        // Do one step
        newMonitor.worked(1);

        try {
        	// Operation steps
        	
			// add concept to taxon
			concept.addTaxonRelation(taxon, taxonRelationshipType, null, null);	
			StoreUtil.isCanceled(newMonitor, 1);
        }
        finally {
        	
        	// Stop the progress monitor.
            newMonitor.done();
        }
		
		// redraw editor if exists
		return postExecute(concept);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#redo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus redo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		return execute(monitor, info);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#undo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus undo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		
		taxon.removeTaxon(concept, taxonRelationshipType);
		
		// redraw editor if exists
		return postExecute(null);
	}
}
