/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.operations;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;

import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.store.StoreUtil;

/**
 * @author p.ciardelli
 * @created 15.01.2009
 * @version 1.0
 */
public class RemoveHomotypicalGroupBasionymOperation extends
		AbstractPostOperation {

	private Synonym synonym;

	public RemoveHomotypicalGroupBasionymOperation(String text,
			IUndoContext undoContext, Taxon taxon, Synonym synonym, IPostOperationEnabled postOperationEnabled) {
		super(text, undoContext, taxon, postOperationEnabled);
		
		this.synonym = synonym;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#execute(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus execute(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		
		// Start the main progress monitor.
        IProgressMonitor newMonitor = StoreUtil.startMainMonitor(monitor,"Removing Basionym from Homotypic Group", 3);
        
        // Do one step
        newMonitor.worked(1);

        try {
        	// Operation steps
	
			TaxonNameBase<?, ?> oldSynonymName = synonym.getName();
			StoreUtil.isCanceled(newMonitor, 1);

			oldSynonymName.removeAsGroupsBasionym();
			StoreUtil.isCanceled(newMonitor, 1);
        }
        finally {
        	
        	// Stop the progress monitor.
            newMonitor.done();
        }
				
		return postExecute(synonym);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#redo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus redo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		return execute(monitor, info);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#undo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus undo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		synonym.getName().setAsGroupsBasionym();
				
		return postExecute(synonym);
	}
}
