/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.operations;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;

import eu.etaxonomy.cdm.api.utility.ImagesUtility;
import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.cdm.model.media.ImageFile;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.store.StoreUtil;

/**
 * @author p.ciardelli
 * @created 05.02.2009
 * @version 1.0
 * @author n.hoffmann
 */
public class RemoveImageFromDescriptionElementOperation extends AbstractPostOperation {
	
	private DescriptionElementBase element;
	private ImageFile image;

	/**
	 * @param name
	 * @param undoContext
	 * @param taxon
	 * @param element
	 * @param postOperationEnabled
	 */
	public RemoveImageFromDescriptionElementOperation(String label,
			IUndoContext undoContext, Taxon taxon, ImageFile image,
			DescriptionElementBase element, IPostOperationEnabled postOperationEnabled) {
		super(label, undoContext, taxon, postOperationEnabled);
		
		this.image = image;
		this.element = element;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#execute(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus execute(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		
		// Start the main progress monitor.
        IProgressMonitor newMonitor = StoreUtil.startMainMonitor(monitor,"Removing Image from Description Element", 2);
        
        // Do one step
        newMonitor.worked(1);

        try {
        	// Operation steps
	
			ImagesUtility.removeImage(element, image);
			StoreUtil.isCanceled(newMonitor, 1);
        }
        finally {
        	
        	// Stop the progress monitor.
            newMonitor.done();
        }
			
		return postExecute(element);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#redo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus redo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		return execute(monitor, info);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#undo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus undo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		image = ImagesUtility.addImagePart(element);
		return postExecute(image);
	}
}
