// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.parser;

import eu.etaxonomy.cdm.model.agent.TeamOrPersonBase;
import eu.etaxonomy.cdm.model.name.NonViralName;
import eu.etaxonomy.cdm.model.reference.ReferenceBase;
import eu.etaxonomy.cdm.strategy.match.DefaultMatchStrategy;
import eu.etaxonomy.cdm.strategy.match.IMatchStrategy;
import eu.etaxonomy.cdm.strategy.match.MatchException;
import eu.etaxonomy.cdm.strategy.match.MatchMode;

/**
 * This class defines the MatchStrategies that will be used by the ParseHandler.
 * 
 * @author n.hoffmann
 * @created Jan 22, 2010
 * @version 1.0
 */
public class MatchStrategyConfigurator {
	
	/**
	 * Singleton instance
	 */
	private static MatchStrategyConfigurator instance = new MatchStrategyConfigurator();
	
	private IMatchStrategy nonViralNameMatchStrategy;
	private IMatchStrategy teamOrPersonMatchStrategy;
	private IMatchStrategy referenceMatchStrategy;
	
	/**
	 * 
	 * @return
	 * @throws MatchException
	 */
	public static IMatchStrategy NonViralNameMatchStrategy() throws MatchException{
		return instance.getNonViralNameMatchStrategy();
	}
	
	/**
	 * 
	 * @return
	 * @throws MatchException
	 */
	public static IMatchStrategy TeamOrPersonMatchStrategy() throws MatchException{
		return instance.getTeamOrPersonMatchStrategy();
	}
	
	/**
	 * 
	 * @return
	 * @throws MatchException
	 */
	public static IMatchStrategy ReferenceMatchStrategy() throws MatchException{
		return instance.getReferenceMatchStrategy();
	}

	/**
	 * @param strategy the nonViralNameMatchStrategy to set
	 */
	private void doSetNonViralNameMatchStrategy(IMatchStrategy strategy) {
		this.nonViralNameMatchStrategy = strategy;
	}
	
	/**
	 * @param strategy the nonViralNameMatchStrategy to set
	 */
	public static void setNonViralNameMatchStrategy(IMatchStrategy strategy) {
		instance.doSetNonViralNameMatchStrategy(strategy);
	}
	
	/**
	 * @param strategy the teamOrPersonMatchStrategy to set
	 */
	private void doSetTeamOrPersonMatchStrategy(IMatchStrategy strategy) {
		this.teamOrPersonMatchStrategy = strategy;
	}

	/**
	 * @param strategy the teamOrPersonMatchStrategy to set
	 */
	public static void setTeamOrPersonMatchStrategy(IMatchStrategy strategy) {
		instance.doSetTeamOrPersonMatchStrategy(strategy);
	}

	/**
	 * @param strategy the referenceMatchStrategy to set
	 */
	private void doSetReferenceMatchStrategy(IMatchStrategy strategy) {
		this.referenceMatchStrategy = strategy;
	}
	
	/**
	 * @param strategy the referenceMatchStrategy to set
	 */
	public static void setReferenceMatchStrategy(IMatchStrategy strategy) {
		instance.doSetReferenceMatchStrategy(strategy);
	}

	/**
	 * @return the nonViralNameMatchStrategy
	 * @throws MatchException 
	 */
	private IMatchStrategy getNonViralNameMatchStrategy() throws MatchException {
		if(nonViralNameMatchStrategy == null){
			nonViralNameMatchStrategy = getDefaultNonViralNameMatchStrategy();
		}
		return nonViralNameMatchStrategy;
	}
	
	/**
	 * 
	 * @return
	 * @throws MatchException
	 */
	public static IMatchStrategy getDefaultNonViralNameMatchStrategy() throws MatchException{
		DefaultMatchStrategy strategy = DefaultMatchStrategy.NewInstance(NonViralName.class);
		
		strategy.setMatchMode("nomenclaturalReference", MatchMode.IGNORE);
		strategy.setMatchMode("combinationAuthorTeam", MatchMode.IGNORE);
		strategy.setMatchMode("exCombinationAuthorTeam", MatchMode.IGNORE);
		strategy.setMatchMode("basionymAuthorTeam", MatchMode.IGNORE);
		strategy.setMatchMode("exBasionymAuthorTeam", MatchMode.IGNORE);
		
		return strategy;
	}

	/**
	 * @return the teamOrPersonMatchStrategy
	 */
	private IMatchStrategy getTeamOrPersonMatchStrategy() throws MatchException{
		if(teamOrPersonMatchStrategy == null){
			teamOrPersonMatchStrategy = getDefaultTeamOrPersonMatchStrategy();
		}
		return teamOrPersonMatchStrategy;
	}
	
	/**
	 * 
	 * @return
	 */
	public static IMatchStrategy getDefaultTeamOrPersonMatchStrategy() throws MatchException{
		IMatchStrategy strategy = DefaultMatchStrategy.NewInstance(TeamOrPersonBase.class);
		
		return strategy;
	}
	

	/**
	 * @return the referenceMatchStrategy
	 */
	private IMatchStrategy getReferenceMatchStrategy() throws MatchException{
		if(referenceMatchStrategy == null){
			referenceMatchStrategy = getDefaultReferenceMatchStrategy();
		}
		return referenceMatchStrategy;
	}
	
	public static IMatchStrategy getDefaultReferenceMatchStrategy() throws MatchException{
		DefaultMatchStrategy strategy = DefaultMatchStrategy.NewInstance(ReferenceBase.class);
		
		return strategy;
	}
	
}
