// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.user.wizard;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.operations.IUndoableOperation;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.wizard.Wizard;

import eu.etaxonomy.cdm.model.common.User;
import eu.etaxonomy.taxeditor.operations.CreateUserOperation;
import eu.etaxonomy.taxeditor.operations.EditUserOperation;
import eu.etaxonomy.taxeditor.store.StoreUtil;
import eu.etaxonomy.taxeditor.user.view.UserManagerView;

/**
 * @author n.hoffmann
 * @created 02.07.2009
 * @version 1.0
 */
public class UserWizard extends Wizard {
	private static final Logger logger = Logger.getLogger(UserWizard.class);
	private User user;
	private UserWizardPage page;
	private UserManagerView view;
	private boolean createMode;

	public UserWizard(UserManagerView view){
		this.view = view;
		setWindowTitle("User Wizard");
		createMode = true;
	}
	
	public UserWizard(UserManagerView view, User user){
		this(view);
		if(user != null){
			this.user = user;
			createMode = false;
		}
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.wizard.Wizard#addPages()
	 */
	@Override
	public void addPages() {
		super.addPages();
		
		page = new UserWizardPage(user);
		
		addPage(page);
		
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.wizard.Wizard#performFinish()
	 */
	@Override
	public boolean performFinish() {
		
		IUndoableOperation operation;
		if(createMode){
			operation = new CreateUserOperation("Create User", 
					StoreUtil.getUndoContext(), page.getUser(), view, view);
		}else{
			operation = new EditUserOperation("Edit User",
					StoreUtil.getUndoContext(), page.getUser(), view, view);
		}
		
		return StoreUtil.executeOperation(operation) == Status.OK_STATUS ? true : false;
	}

}
